
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS
/* Enumerations from "eog-debug.h" */
#define EOG_TYPE_DEBUG	(eog_debug_get_type())
GType eog_debug_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-enums.h" */
#define EOG_TYPE_IMAGE_DATA	(eog_image_data_get_type())
GType eog_image_data_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-image.h" */
#define EOG_TYPE_IMAGE_ERROR	(eog_image_error_get_type())
GType eog_image_error_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_IMAGE_STATUS	(eog_image_status_get_type())
GType eog_image_status_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_IMAGE_METADATA_STATUS	(eog_image_metadata_status_get_type())
GType eog_image_metadata_status_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-job-scheduler.h" */
#define EOG_TYPE_JOB_PRIORITY	(eog_job_priority_get_type())
GType eog_job_priority_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-list-store.h" */
#define EOG_TYPE_LIST_STORE_COLUMN	(eog_list_store_column_get_type())
GType eog_list_store_column_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-scroll-view.h" */
#define EOG_TYPE_TRANSPARENCY_STYLE	(eog_transparency_style_get_type())
GType eog_transparency_style_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_ZOOM_MODE	(eog_zoom_mode_get_type())
GType eog_zoom_mode_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-thumb-nav.h" */
#define EOG_TYPE_THUMB_NAV_MODE	(eog_thumb_nav_mode_get_type())
GType eog_thumb_nav_mode_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-thumb-view.h" */
#define EOG_TYPE_THUMB_VIEW_SELECTION_CHANGE	(eog_thumb_view_selection_change_get_type())
GType eog_thumb_view_selection_change_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-transform.h" */
#define EOG_TYPE_TRANSFORM_TYPE	(eog_transform_type_get_type())
GType eog_transform_type_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-uri-converter.h" */
#define EOG_TYPE_UC_TYPE	(eog_uc_type_get_type())
GType eog_uc_type_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_UC_ERROR	(eog_uc_error_get_type())
GType eog_uc_error_get_type	(void) G_GNUC_CONST;
/* Enumerations from "eog-window.h" */
#define EOG_TYPE_WINDOW_MODE	(eog_window_mode_get_type())
GType eog_window_mode_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_WINDOW_GALLERY_POS	(eog_window_gallery_pos_get_type())
GType eog_window_gallery_pos_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_WINDOW_ERROR	(eog_window_error_get_type())
GType eog_window_error_get_type	(void) G_GNUC_CONST;
#define EOG_TYPE_STARTUP_FLAGS	(eog_startup_flags_get_type())
GType eog_startup_flags_get_type	(void) G_GNUC_CONST;
G_END_DECLS

/* Generated data ends here */

